# Copyright 2013 IBM Corp.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.

import six

from pycadf import cadftype

TYPE_URI_REASON = cadftype.CADF_VERSION_1_0_0 + 'reason'

REASON_KEYNAME_REASONTYPE = "reasonType"
REASON_KEYNAME_REASONCODE = "reasonCode"
REASON_KEYNAME_POLICYTYPE = "policyType"
REASON_KEYNAME_POLICYID = "policyId"

REASON_KEYNAMES = [REASON_KEYNAME_REASONTYPE,
                   REASON_KEYNAME_REASONCODE,
                   REASON_KEYNAME_POLICYTYPE,
                   REASON_KEYNAME_POLICYID]


class Reason(cadftype.CADFAbstractType):

    reasonType = cadftype.ValidatorDescriptor(
        REASON_KEYNAME_REASONTYPE,
        lambda x: isinstance(x, six.string_types))
    reasonCode = cadftype.ValidatorDescriptor(
        REASON_KEYNAME_REASONCODE,
        lambda x: isinstance(x, six.string_types))
    policyType = cadftype.ValidatorDescriptor(
        REASON_KEYNAME_POLICYTYPE,
        lambda x: isinstance(x, six.string_types))
    policyId = cadftype.ValidatorDescriptor(
        REASON_KEYNAME_POLICYID,
        lambda x: isinstance(x, six.string_types))

    def __init__(self, reasonType=None, reasonCode=None, policyType=None,
                 policyId=None):
        """Create Reason data type

        :param reasonType: domain URI which describes reasonCode
        :param reasonCode: detailed result code
        :param policyType: domain URI which describes policyId
        :param policyId: id of policy applied that describes outcome
        """

        # Reason.reasonType
        if reasonType is not None:
            setattr(self, REASON_KEYNAME_REASONTYPE, reasonType)

        # Reason.reasonCode
        if reasonCode is not None:
            setattr(self, REASON_KEYNAME_REASONCODE, reasonCode)

        # Reason.policyType
        if policyType is not None:
            setattr(self, REASON_KEYNAME_POLICYTYPE, policyType)

        # Reason.policyId
        if policyId is not None:
            setattr(self, REASON_KEYNAME_POLICYID, policyId)

    # TODO(mrutkows): validate this cadf:Reason type against schema
    def is_valid(self):
        """Validation to ensure Reason required attributes are set.
        """
        # MUST have at least one valid pairing of reason+code or policy+id
        return ((self._isset(REASON_KEYNAME_REASONTYPE) and
                 self._isset(REASON_KEYNAME_REASONCODE)) or
                (self._isset(REASON_KEYNAME_POLICYTYPE) and
                 self._isset(REASON_KEYNAME_POLICYID)))



# Copyright 2013 IBM Corp.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.

import six

from pycadf import cadftype

ATTACHMENT_KEYNAME_TYPEURI = "typeURI"
ATTACHMENT_KEYNAME_CONTENT = "content"
ATTACHMENT_KEYNAME_NAME = "name"

ATTACHMENT_KEYNAMES = [ATTACHMENT_KEYNAME_TYPEURI,
                       ATTACHMENT_KEYNAME_CONTENT,
                       ATTACHMENT_KEYNAME_NAME]


class Attachment(cadftype.CADFAbstractType):

    # TODO(mrutkows): OpenStack / Ceilometer may want to define
    # the set of approved attachment types in order to
    # limit and validate them.
    typeURI = cadftype.ValidatorDescriptor(ATTACHMENT_KEYNAME_TYPEURI,
                                           lambda x: isinstance(
                                               x, six.string_types))
    content = cadftype.ValidatorDescriptor(ATTACHMENT_KEYNAME_CONTENT)
    name = cadftype.ValidatorDescriptor(ATTACHMENT_KEYNAME_NAME,
                                        lambda x: isinstance(x,
                                                             six.string_types))

    def __init__(self, typeURI=None, content=None, name=None):
        """Create Attachment data type

        :param typeURI: uri that identifies type of data in content
        :param content: container that contains any type of data
        :param contentType: name used to identify content.
        """
        # Attachment.typeURI
        if typeURI is not None:
            setattr(self, ATTACHMENT_KEYNAME_TYPEURI, typeURI)

        # Attachment.content
        if content is not None:
            setattr(self, ATTACHMENT_KEYNAME_CONTENT, content)

        # Attachment.name
        if name is not None:
            setattr(self, ATTACHMENT_KEYNAME_NAME, name)

    # self validate cadf:Attachment type against schema
    def is_valid(self):
        """Validation to ensure Attachment required attributes are set.
        """
        return (
            self._isset(ATTACHMENT_KEYNAME_TYPEURI) and
            self._isset(ATTACHMENT_KEYNAME_NAME) and
            self._isset(ATTACHMENT_KEYNAME_CONTENT)
        )



# Copyright 2013 IBM Corp.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.

from pycadf import cadftype
from pycadf import identifier
from pycadf import resource
from pycadf import timestamp

REPORTERSTEP_KEYNAME_ROLE = "role"
REPORTERSTEP_KEYNAME_REPORTER = "reporter"
REPORTERSTEP_KEYNAME_REPORTERID = "reporterId"
REPORTERSTEP_KEYNAME_REPORTERTIME = "reporterTime"
# REPORTERSTEP_KEYNAME_ATTACHMENTS = "attachments"

REPORTERSTEP_KEYNAMES = [REPORTERSTEP_KEYNAME_ROLE,
                         REPORTERSTEP_KEYNAME_REPORTER,
                         REPORTERSTEP_KEYNAME_REPORTERID,
                         REPORTERSTEP_KEYNAME_REPORTERTIME,
                         # REPORTERSTEP_KEYNAME_ATTACHMENTS
                         ]


class Reporterstep(cadftype.CADFAbstractType):

    role = cadftype.ValidatorDescriptor(
        REPORTERSTEP_KEYNAME_ROLE,
        lambda x: cadftype.is_valid_reporter_role(x))
    reporter = cadftype.ValidatorDescriptor(
        REPORTERSTEP_KEYNAME_REPORTER,
        (lambda x: isinstance(x, resource.Resource) and x.is_valid()))
    reporterId = cadftype.ValidatorDescriptor(
        REPORTERSTEP_KEYNAME_REPORTERID, lambda x: identifier.is_valid(x))
    reporterTime = cadftype.ValidatorDescriptor(
        REPORTERSTEP_KEYNAME_REPORTERTIME, lambda x: timestamp.is_valid(x))

    def __init__(self, role=cadftype.REPORTER_ROLE_MODIFIER,
                 reporterTime=None, reporter=None, reporterId=None):
        """Create ReporterStep data type

        :param role: optional role of Reporterstep. Defaults to 'modifier'
        :param reporterTime: utc time of Reporterstep.
        :param reporter: CADF Resource of reporter
        :param reporterId: id of CADF resource for reporter
        """
        # Reporterstep.role
        setattr(self, REPORTERSTEP_KEYNAME_ROLE, role)

        # Reporterstep.reportTime
        if reporterTime is not None:
            setattr(self, REPORTERSTEP_KEYNAME_REPORTERTIME, reporterTime)

        # Reporterstep.reporter
        if reporter is not None:
            setattr(self, REPORTERSTEP_KEYNAME_REPORTER, reporter)

        # Reporterstep.reporterId
        if reporterId is not None:
            setattr(self, REPORTERSTEP_KEYNAME_REPORTERID, reporterId)

    # self validate this cadf:Reporterstep type against schema
    def is_valid(self):
        """Validation to ensure Reporterstep required attributes are set.
        """
        return (
            self._isset(REPORTERSTEP_KEYNAME_ROLE) and
            (self._isset(REPORTERSTEP_KEYNAME_REPORTER) ^
             self._isset(REPORTERSTEP_KEYNAME_REPORTERID))
        )



# Copyright (c) 2013 IBM Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.

import six

from pycadf import cadftype
from pycadf import identifier

TYPE_URI_HOST = cadftype.CADF_VERSION_1_0_0 + 'host'

HOST_KEYNAME_ID = "id"
HOST_KEYNAME_ADDR = "address"
HOST_KEYNAME_AGENT = "agent"
HOST_KEYNAME_PLATFORM = "platform"

HOST_KEYNAMES = [HOST_KEYNAME_ID,
                 HOST_KEYNAME_ADDR,
                 HOST_KEYNAME_AGENT,
                 HOST_KEYNAME_PLATFORM]


class Host(cadftype.CADFAbstractType):

    id = cadftype.ValidatorDescriptor(
        HOST_KEYNAME_ID, lambda x: identifier.is_valid(x))
    address = cadftype.ValidatorDescriptor(
        HOST_KEYNAME_ADDR, lambda x: isinstance(x, six.string_types))
    agent = cadftype.ValidatorDescriptor(
        HOST_KEYNAME_AGENT, lambda x: isinstance(x, six.string_types))
    platform = cadftype.ValidatorDescriptor(
        HOST_KEYNAME_PLATFORM, lambda x: isinstance(x, six.string_types))

    def __init__(self, id=None, address=None, agent=None,
                 platform=None):
        """Create Host data type

        :param id: id of Host
        :param address: optional Address of Host
        :param agent: agent (name) of Host
        :param platform: platform of Host
        """

        # Host.id
        if id is not None:
            setattr(self, HOST_KEYNAME_ID, id)
        # Host.address
        if address is not None:
            setattr(self, HOST_KEYNAME_ADDR, address)
        # Host.agent
        if agent is not None:
            setattr(self, HOST_KEYNAME_AGENT, agent)
        # Host.platform
        if platform is not None:
            setattr(self, HOST_KEYNAME_PLATFORM, platform)

    # TODO(mrutkows): validate this cadf:Host type against schema
    def is_valid(self):
        """Validation to ensure Host required attributes are set.
        """
        return True



# Copyright 2013 IBM Corp.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.

from pycadf import cadftype

TYPE_URI_ACTION = cadftype.CADF_VERSION_1_0_0 + 'action'

UNKNOWN = 'unknown'

# Commonly used (valid) Event.action values from Nova
ACTION_CREATE = 'create'
ACTION_READ = 'read'
ACTION_UPDATE = 'update'
ACTION_DELETE = 'delete'
# Other CADF actions
ACTION_AUTHENTICATE = 'authenticate'
ACTION_EVALUATE = 'evaluate'
# OpenStack specific, Profile or change CADF spec. to add this action
ACTION_LIST = 'read/list'

# TODO(mrutkows): Make global using WSGI mechanism
ACTION_TAXONOMY = frozenset([
    'backup',
    'capture',
    ACTION_CREATE,
    'configure',
    ACTION_READ,
    ACTION_LIST,
    ACTION_UPDATE,
    ACTION_DELETE,
    'monitor',
    'start',
    'stop',
    'deploy',
    'undeploy',
    'enable',
    'disable',
    'send',
    'receive',
    ACTION_AUTHENTICATE,
    'authenticate/login',
    'revoke',
    'renew',
    'restore',
    ACTION_EVALUATE,
    'allow',
    'deny',
    'notify',
    UNKNOWN
])


# TODO(mrutkows): validate absolute URIs as well
def is_valid_action(value):
    for type in ACTION_TAXONOMY:
        if value.startswith(type):
            return True
    return False


TYPE_URI_OUTCOME = cadftype.CADF_VERSION_1_0_0 + 'outcome'

# Valid Event.outcome values
OUTCOME_SUCCESS = 'success'
OUTCOME_FAILURE = 'failure'
OUTCOME_PENDING = 'pending'

# TODO(mrutkows): Make global using WSGI mechanism
OUTCOME_TAXONOMY = frozenset([
    OUTCOME_SUCCESS,
    OUTCOME_FAILURE,
    OUTCOME_PENDING,
    UNKNOWN
])


# TODO(mrutkows): validate absolute URIs as well
def is_valid_outcome(value):
    return value in OUTCOME_TAXONOMY

SERVICE_SECURITY = 'service/security'
SERVICE_KEYMGR = 'service/security/keymanager'
ACCOUNT_USER = 'service/security/account/user'
CADF_AUDIT_FILTER = 'service/security/audit/filter'

SECURITY_ACCOUNT = 'data/security/account'
SECURITY_CREDENTIAL = 'data/security/credential'
SECURITY_DOMAIN = 'data/security/domain'
SECURITY_ENDPOINT = 'data/security/endpoint'
SECURITY_GROUP = 'data/security/group'
SECURITY_IDENTITY = 'data/security/identity'
SECURITY_KEY = 'data/security/key'
SECURITY_LICENCE = 'data/security/license'
SECURITY_POLICY = 'data/security/policy'
SECURITY_PROFILE = 'data/security/profile'
SECURITY_PROJECT = 'data/security/project'
SECURITY_REGION = 'data/security/region'
SECURITY_ROLE = 'data/security/role'
SECURITY_SERVICE = 'data/security/service'
SECURITY_TRUST = 'data/security/trust'
SECURITY_ACCOUNT_USER = 'data/security/account/user'
KEYMGR_SECRET = 'data/security/keymanager/secret'
KEYMGR_CONTAINER = 'data/security/keymanager/container'
KEYMGR_ORDER = 'data/security/keymanager/order'
KEYMGR_OTHERS = 'data/security/keymanager'


# TODO(mrutkows): Make global using WSGI mechanism
RESOURCE_TAXONOMY = frozenset([
    'storage',
    'storage/node',
    'storage/volume',
    'storage/memory',
    'storage/container',
    'storage/directory',
    'storage/database',
    'storage/queue',
    'compute',
    'compute/node',
    'compute/cpu',
    'compute/machine',
    'compute/process',
    'compute/thread',
    'network',
    'network/node',
    'network/node/host',
    'network/connection',
    'network/domain',
    'network/cluster',
    'service',
    'service/oss',
    'service/bss',
    'service/bss/metering',
    'service/composition',
    'service/compute',
    'service/database',
    SERVICE_SECURITY,
    SERVICE_KEYMGR,
    'service/security/account',
    ACCOUNT_USER,
    CADF_AUDIT_FILTER,
    'service/storage',
    'service/storage/block',
    'service/storage/image',
    'service/storage/object',
    'service/network',
    'data',
    'data/message',
    'data/workload',
    'data/workload/app',
    'data/workload/service',
    'data/workload/task',
    'data/workload/job',
    'data/file',
    'data/file/catalog',
    'data/file/log',
    'data/template',
    'data/package',
    'data/image',
    'data/module',
    'data/config',
    'data/directory',
    'data/database',
    'data/security',
    SECURITY_ACCOUNT,
    SECURITY_CREDENTIAL,
    SECURITY_DOMAIN,
    SECURITY_ENDPOINT,
    SECURITY_GROUP,
    SECURITY_IDENTITY,
    SECURITY_KEY,
    SECURITY_LICENCE,
    SECURITY_POLICY,
    SECURITY_PROFILE,
    SECURITY_PROJECT,
    SECURITY_REGION,
    SECURITY_ROLE,
    SECURITY_SERVICE,
    SECURITY_TRUST,
    SECURITY_ACCOUNT_USER,
    'data/security/account/user/privilege',
    'data/database/alias',
    'data/database/catalog',
    'data/database/constraints',
    'data/database/index',
    'data/database/instance',
    'data/database/key',
    'data/database/routine',
    'data/database/schema',
    'data/database/sequence',
    'data/database/table',
    'data/database/trigger',
    'data/database/view',
    KEYMGR_CONTAINER,
    KEYMGR_ORDER,
    KEYMGR_SECRET,
    KEYMGR_OTHERS,
    UNKNOWN
])


# TODO(mrutkows): validate absolute URIs as well
def is_valid_resource(value):
    for type in RESOURCE_TAXONOMY:
        if value.startswith(type):
            return True
    return False



# Copyright 2013 IBM Corp.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.

import six

from pycadf import attachment
from pycadf import cadftaxonomy
from pycadf import cadftype
from pycadf import identifier
from pycadf import measurement
from pycadf import reason
from pycadf import reporterstep
from pycadf import resource
from pycadf import tag
from pycadf import timestamp

TYPE_URI_EVENT = cadftype.CADF_VERSION_1_0_0 + 'event'

# Event.eventType
EVENT_KEYNAME_TYPEURI = "typeURI"
EVENT_KEYNAME_EVENTTYPE = "eventType"
EVENT_KEYNAME_ID = "id"
EVENT_KEYNAME_EVENTTIME = "eventTime"
EVENT_KEYNAME_INITIATOR = "initiator"
EVENT_KEYNAME_INITIATORID = "initiatorId"
EVENT_KEYNAME_ACTION = "action"
EVENT_KEYNAME_TARGET = "target"
EVENT_KEYNAME_TARGETID = "targetId"
EVENT_KEYNAME_OUTCOME = "outcome"
EVENT_KEYNAME_REASON = "reason"
EVENT_KEYNAME_SEVERITY = "severity"
EVENT_KEYNAME_NAME = "name"
EVENT_KEYNAME_MEASUREMENTS = "measurements"
EVENT_KEYNAME_TAGS = "tags"
EVENT_KEYNAME_ATTACHMENTS = "attachments"
EVENT_KEYNAME_OBSERVER = "observer"
EVENT_KEYNAME_OBSERVERID = "observerId"
EVENT_KEYNAME_REPORTERCHAIN = "reporterchain"

EVENT_KEYNAMES = [EVENT_KEYNAME_TYPEURI,
                  EVENT_KEYNAME_EVENTTYPE,
                  EVENT_KEYNAME_ID,
                  EVENT_KEYNAME_EVENTTIME,
                  EVENT_KEYNAME_INITIATOR,
                  EVENT_KEYNAME_INITIATORID,
                  EVENT_KEYNAME_ACTION,
                  EVENT_KEYNAME_TARGET,
                  EVENT_KEYNAME_TARGETID,
                  EVENT_KEYNAME_OUTCOME,
                  EVENT_KEYNAME_REASON,
                  EVENT_KEYNAME_SEVERITY,
                  EVENT_KEYNAME_NAME,
                  EVENT_KEYNAME_MEASUREMENTS,
                  EVENT_KEYNAME_TAGS,
                  EVENT_KEYNAME_ATTACHMENTS,
                  EVENT_KEYNAME_OBSERVER,
                  EVENT_KEYNAME_OBSERVERID,
                  EVENT_KEYNAME_REPORTERCHAIN]


class Event(cadftype.CADFAbstractType):

    eventType = cadftype.ValidatorDescriptor(
        EVENT_KEYNAME_EVENTTYPE, lambda x: cadftype.is_valid_eventType(x))
    id = cadftype.ValidatorDescriptor(EVENT_KEYNAME_ID,
                                      lambda x: identifier.is_valid(x))
    eventTime = cadftype.ValidatorDescriptor(EVENT_KEYNAME_EVENTTIME,
                                             lambda x: timestamp.is_valid(x))
    initiator = cadftype.ValidatorDescriptor(
        EVENT_KEYNAME_INITIATOR,
        (lambda x: isinstance(x, resource.Resource) and x.is_valid()
         and x.id != 'initiator'))
    initiatorId = cadftype.ValidatorDescriptor(
        EVENT_KEYNAME_INITIATORID, lambda x: identifier.is_valid(x))
    action = cadftype.ValidatorDescriptor(
        EVENT_KEYNAME_ACTION, lambda x: cadftaxonomy.is_valid_action(x))
    target = cadftype.ValidatorDescriptor(
        EVENT_KEYNAME_TARGET,
        (lambda x: isinstance(x, resource.Resource) and x.is_valid()
         and x.id != 'target'))
    targetId = cadftype.ValidatorDescriptor(
        EVENT_KEYNAME_TARGETID, lambda x: identifier.is_valid(x))
    outcome = cadftype.ValidatorDescriptor(
        EVENT_KEYNAME_OUTCOME, lambda x: cadftaxonomy.is_valid_outcome(x))
    reason = cadftype.ValidatorDescriptor(
        EVENT_KEYNAME_REASON,
        lambda x: isinstance(x, reason.Reason) and x.is_valid())
    name = cadftype.ValidatorDescriptor(EVENT_KEYNAME_NAME,
                                        lambda x: isinstance(
                                            x, six.string_types))
    severity = cadftype.ValidatorDescriptor(EVENT_KEYNAME_SEVERITY,
                                            lambda x: isinstance(
                                                x, six.string_types))
    observer = cadftype.ValidatorDescriptor(
        EVENT_KEYNAME_OBSERVER,
        (lambda x: isinstance(x, resource.Resource) and x.is_valid()))
    observerId = cadftype.ValidatorDescriptor(
        EVENT_KEYNAME_OBSERVERID, lambda x: identifier.is_valid(x))

    def __init__(self, eventType=cadftype.EVENTTYPE_ACTIVITY,
                 id=None, eventTime=None,
                 action=cadftaxonomy.UNKNOWN, outcome=cadftaxonomy.UNKNOWN,
                 initiator=None, initiatorId=None, target=None, targetId=None,
                 severity=None, reason=None, observer=None, observerId=None,
                 name=None):
        """Create an Event

        :param eventType: eventType of Event. Defaults to 'activity' type
        :param id: id of event. will generate uuid if None
        :param eventTime: time of event. will take current utc if None
        :param action: event's action (see Action taxonomy)
        :param outcome: Event's outcome (see Outcome taxonomy)
        :param initiator: Event's Initiator Resource
        :param initiatorId: Event's Initiator Resource id
        :param target: Event's Target Resource
        :param targetId: Event's Target Resource id
        :param severity: domain-relative severity of Event
        :param reason: domain-specific Reason type
        :param observer: Event's Observer Resource
        :param observerId: Event's Observer Resource id
        :param name: descriptive name for the event
        """
        # Establish typeURI for the CADF Event data type
        # TODO(mrutkows): support extended typeURIs for Event subtypes
        setattr(self, EVENT_KEYNAME_TYPEURI, TYPE_URI_EVENT)

        # Event.eventType (Mandatory)
        setattr(self, EVENT_KEYNAME_EVENTTYPE, eventType)

        # Event.id (Mandatory)
        setattr(self, EVENT_KEYNAME_ID, id or identifier.generate_uuid())

        # Event.eventTime (Mandatory)
        setattr(self, EVENT_KEYNAME_EVENTTIME,
                eventTime or timestamp.get_utc_now())

        # Event.action (Mandatory)
        setattr(self, EVENT_KEYNAME_ACTION, action)

        # Event.outcome (Mandatory)
        setattr(self, EVENT_KEYNAME_OUTCOME, outcome)

        # Event.observer (Mandatory if no observerId)
        if observer is not None:
            setattr(self, EVENT_KEYNAME_OBSERVER, observer)
        # Event.observerId (Dependent)
        if observerId is not None:
            setattr(self, EVENT_KEYNAME_OBSERVERID, observerId)

        # Event.initiator (Mandatory if no initiatorId)
        if initiator is not None:
            setattr(self, EVENT_KEYNAME_INITIATOR, initiator)
        # Event.initiatorId (Dependent)
        if initiatorId is not None:
            setattr(self, EVENT_KEYNAME_INITIATORID, initiatorId)

        # Event.target (Mandatory if no targetId)
        if target is not None:
            setattr(self, EVENT_KEYNAME_TARGET, target)
        # Event.targetId (Dependent)
        if targetId is not None:
            setattr(self, EVENT_KEYNAME_TARGETID, targetId)

        # Event.name (Optional)
        if name is not None:
            setattr(self, EVENT_KEYNAME_NAME, name)

        # Event.severity (Optional)
        if severity is not None:
            setattr(self, EVENT_KEYNAME_SEVERITY, severity)

        # Event.reason (Optional)
        if reason is not None:
            setattr(self, EVENT_KEYNAME_REASON, reason)

    # Event.reporterchain
    def add_reporterstep(self, step):
        """Add a Reporterstep

        :param step: Reporterstep to be added to reporterchain
        """
        if step is not None and isinstance(step, reporterstep.Reporterstep):
            if step.is_valid():
                # Create the list of Reportersteps if needed
                if not hasattr(self, EVENT_KEYNAME_REPORTERCHAIN):
                    setattr(self, EVENT_KEYNAME_REPORTERCHAIN, list())

                reporterchain = getattr(self,
                                        EVENT_KEYNAME_REPORTERCHAIN)
                reporterchain.append(step)
            else:
                raise ValueError('Invalid reporterstep')
        else:
            raise ValueError('Invalid reporterstep. '
                             'Value must be a Reporterstep')

    # Event.measurements
    def add_measurement(self, measure_val):
        """Add a measurement value

        :param measure_val: Measurement data type to be added to Event
        """
        if (measure_val is not None
                and isinstance(measure_val, measurement.Measurement)):

            if measure_val.is_valid():

                # Create the list of event.Measurements if needed
                if not hasattr(self, EVENT_KEYNAME_MEASUREMENTS):
                    setattr(self, EVENT_KEYNAME_MEASUREMENTS, list())

                measurements = getattr(self, EVENT_KEYNAME_MEASUREMENTS)
                measurements.append(measure_val)
            else:
                raise ValueError('Invalid measurement')
        else:
            raise ValueError('Invalid measurement. '
                             'Value must be a Measurement')

    # Event.tags
    def add_tag(self, tag_val):
        """Add Tag to Event

        :param tag_val: Tag to add to event
        """
        if tag.is_valid(tag_val):
            if not hasattr(self, EVENT_KEYNAME_TAGS):
                setattr(self, EVENT_KEYNAME_TAGS, list())
            getattr(self, EVENT_KEYNAME_TAGS).append(tag_val)
        else:
            raise ValueError('Invalid tag')

    # Event.attachments
    def add_attachment(self, attachment_val):
        """Add Attachment to Event

        :param attachment_val: Attachment to add to Event
        """
        if (attachment_val is not None
                and isinstance(attachment_val, attachment.Attachment)):

            if attachment_val.is_valid():
                # Create the list of Attachments if needed
                if not hasattr(self, EVENT_KEYNAME_ATTACHMENTS):
                    setattr(self, EVENT_KEYNAME_ATTACHMENTS, list())

                attachments = getattr(self, EVENT_KEYNAME_ATTACHMENTS)
                attachments.append(attachment_val)
            else:
                raise ValueError('Invalid attachment')
        else:
            raise ValueError('Invalid attachment. '
                             'Value must be an Attachment')

    # self validate cadf:Event record against schema
    def is_valid(self):
        """Validation to ensure Event required attributes are set.
        """
        # TODO(mrutkows): Eventually, make sure all attributes are
        # from either the CADF spec. (or profiles thereof)
        # TODO(mrutkows): validate all child attributes that are CADF types
        return (
            self._isset(EVENT_KEYNAME_TYPEURI) and
            self._isset(EVENT_KEYNAME_EVENTTYPE) and
            self._isset(EVENT_KEYNAME_ID) and
            self._isset(EVENT_KEYNAME_EVENTTIME) and
            self._isset(EVENT_KEYNAME_ACTION) and
            self._isset(EVENT_KEYNAME_OUTCOME) and
            (self._isset(EVENT_KEYNAME_INITIATOR) ^
             self._isset(EVENT_KEYNAME_INITIATORID)) and
            (self._isset(EVENT_KEYNAME_TARGET) ^
             self._isset(EVENT_KEYNAME_TARGETID)) and
            (self._isset(EVENT_KEYNAME_OBSERVER) ^
             self._isset(EVENT_KEYNAME_OBSERVERID))
        )



# Copyright 2013 OpenStack LLC
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.

import time
import uuid

from pycadf import attachment
from pycadf import cadftype
from pycadf import credential
from pycadf import endpoint
from pycadf import event
from pycadf import geolocation
from pycadf import host
from pycadf import identifier
from pycadf import measurement
from pycadf import metric
from pycadf import reason
from pycadf import reporterstep
from pycadf import resource
from pycadf import tag
from pycadf.tests import base
from pycadf import timestamp


class TestCADFSpec(base.TestCase):

    def test_identifier_empty(self):
        self.assertFalse(identifier.is_valid(''))
        self.assertTrue(identifier.is_valid(identifier.generate_uuid()))

    def test_endpoint(self):
        endp = endpoint.Endpoint(url='http://192.168.0.1',
                                 name='endpoint name',
                                 port='8080')
        self.assertEqual(endp.is_valid(), True)
        dict_endp = endp.as_dict()
        for key in endpoint.ENDPOINT_KEYNAMES:
            self.assertIn(key, dict_endp)

    def test_host(self):
        h = host.Host(id=identifier.generate_uuid(),
                      address='192.168.0.1',
                      agent='client',
                      platform='AIX')
        self.assertEqual(h.is_valid(), True)
        dict_host = h.as_dict()
        for key in host.HOST_KEYNAMES:
            self.assertIn(key, dict_host)

    def test_credential(self):
        cred = credential.Credential(type='auth token',
                                     token=identifier.generate_uuid())
        self.assertEqual(cred.is_valid(), True)
        dict_cred = cred.as_dict()
        for key in credential.CRED_KEYNAMES:
            self.assertIn(key, dict_cred)

    def test_federated_credential(self):
        cred = credential.FederatedCredential(
            token=identifier.generate_uuid(),
            type='http://docs.oasis-open.org/security/saml/v2.0',
            identity_provider=identifier.generate_uuid(),
            user=identifier.generate_uuid(),
            groups=[
                identifier.generate_uuid(),
                identifier.generate_uuid(),
                identifier.generate_uuid()])
        self.assertEqual(cred.is_valid(), True)
        dict_cred = cred.as_dict()
        for key in credential.FED_CRED_KEYNAMES:
            self.assertIn(key, dict_cred)

    def test_geolocation(self):
        geo = geolocation.Geolocation(id=identifier.generate_uuid(),
                                      latitude='43.6481 N',
                                      longitude='79.4042 W',
                                      elevation='0',
                                      accuracy='1',
                                      city='toronto',
                                      state='ontario',
                                      regionICANN='ca')
        self.assertEqual(geo.is_valid(), True)

        dict_geo = geo.as_dict()
        for key in geolocation.GEO_KEYNAMES:
            self.assertIn(key, dict_geo)

    def test_metric(self):
        metric_val = metric.Metric(metricId=identifier.generate_uuid(),
                                   unit='b',
                                   name='bytes')
        self.assertEqual(metric_val.is_valid(), True)

        dict_metric_val = metric_val.as_dict()
        for key in metric.METRIC_KEYNAMES:
            self.assertIn(key, dict_metric_val)

    def test_measurement(self):
        measure_val = measurement.Measurement(
            result='100',
            metric=metric.Metric(),
            metricId=identifier.generate_uuid(),
            calculatedBy=resource.Resource(typeURI='storage'))
        self.assertEqual(measure_val.is_valid(), False)

        dict_measure_val = measure_val.as_dict()
        for key in measurement.MEASUREMENT_KEYNAMES:
            self.assertIn(key, dict_measure_val)

        measure_val = measurement.Measurement(
            result='100',
            metric=metric.Metric(),
            calculatedBy=resource.Resource(typeURI='storage'))
        self.assertEqual(measure_val.is_valid(), True)

        measure_val = measurement.Measurement(
            result='100',
            metricId=identifier.generate_uuid(),
            calculatedBy=resource.Resource(typeURI='storage'))
        self.assertEqual(measure_val.is_valid(), True)

    def test_reason(self):
        reason_val = reason.Reason(reasonType='HTTP',
                                   reasonCode='200',
                                   policyType='poltype',
                                   policyId=identifier.generate_uuid())
        self.assertEqual(reason_val.is_valid(), True)

        dict_reason_val = reason_val.as_dict()
        for key in reason.REASON_KEYNAMES:
            self.assertIn(key, dict_reason_val)

    def test_reporterstep(self):
        step = reporterstep.Reporterstep(
            role='modifier',
            reporter=resource.Resource(typeURI='storage'),
            reporterId=identifier.generate_uuid(),
            reporterTime=timestamp.get_utc_now())
        self.assertEqual(step.is_valid(), False)

        dict_step = step.as_dict()
        for key in reporterstep.REPORTERSTEP_KEYNAMES:
            self.assertIn(key, dict_step)

        step = reporterstep.Reporterstep(
            role='modifier',
            reporter=resource.Resource(typeURI='storage'),
            reporterTime=timestamp.get_utc_now())
        self.assertEqual(step.is_valid(), True)

        step = reporterstep.Reporterstep(
            role='modifier',
            reporterId=identifier.generate_uuid(),
            reporterTime=timestamp.get_utc_now())
        self.assertEqual(step.is_valid(), True)

    def test_attachment(self):
        attach = attachment.Attachment(typeURI='attachURI',
                                       content='content',
                                       name='attachment_name')
        self.assertEqual(attach.is_valid(), True)

        dict_attach = attach.as_dict()
        for key in attachment.ATTACHMENT_KEYNAMES:
            self.assertIn(key, dict_attach)

    def test_resource(self):
        res = resource.Resource(typeURI='storage',
                                name='res_name',
                                domain='res_domain',
                                ref='res_ref',
                                credential=credential.Credential(
                                    token=identifier.generate_uuid()),
                                host=host.Host(address='192.168.0.1'),
                                geolocation=geolocation.Geolocation(),
                                geolocationId=identifier.generate_uuid())

        res.add_attachment(attachment.Attachment(typeURI='attachURI',
                                                 content='content',
                                                 name='attachment_name'))
        res.add_address(endpoint.Endpoint(url='http://192.168.0.1'))

        self.assertEqual(res.is_valid(), True)
        dict_res = res.as_dict()
        for key in resource.RESOURCE_KEYNAMES:
            self.assertIn(key, dict_res)

    def test_resource_shortform(self):
        res = resource.Resource(id='target')
        self.assertEqual(res.is_valid(), True)

        res.add_attachment(attachment.Attachment(typeURI='attachURI',
                                                 content='content',
                                                 name='attachment_name'))
        self.assertEqual(res.is_valid(), False)

    def test_event(self):
        ev = event.Event(eventType='activity',
                         id=identifier.generate_uuid(),
                         eventTime=timestamp.get_utc_now(),
                         initiator=resource.Resource(typeURI='storage'),
                         initiatorId=identifier.generate_uuid(),
                         action='read',
                         target=resource.Resource(typeURI='storage'),
                         targetId=identifier.generate_uuid(),
                         observer=resource.Resource(id='target'),
                         observerId=identifier.generate_uuid(),
                         outcome='success',
                         reason=reason.Reason(reasonType='HTTP',
                                              reasonCode='200'),
                         severity='high',
                         name='descriptive name')
        ev.add_measurement(
            measurement.Measurement(result='100',
                                    metricId=identifier.generate_uuid())),
        ev.add_tag(tag.generate_name_value_tag('name', 'val'))
        ev.add_attachment(attachment.Attachment(typeURI='attachURI',
                                                content='content',
                                                name='attachment_name'))
        ev.observer = resource.Resource(typeURI='service/security')
        ev.add_reporterstep(reporterstep.Reporterstep(
            role='observer',
            reporter=resource.Resource(typeURI='service/security')))
        ev.add_reporterstep(reporterstep.Reporterstep(
            reporterId=identifier.generate_uuid()))
        self.assertEqual(ev.is_valid(), False)

        dict_ev = ev.as_dict()
        for key in event.EVENT_KEYNAMES:
            self.assertIn(key, dict_ev)

        ev = event.Event(eventType='activity',
                         id=identifier.generate_uuid(),
                         eventTime=timestamp.get_utc_now(),
                         initiator=resource.Resource(typeURI='storage'),
                         action='read',
                         target=resource.Resource(typeURI='storage'),
                         observer=resource.Resource(id='target'),
                         outcome='success')
        self.assertEqual(ev.is_valid(), True)

        ev = event.Event(eventType='activity',
                         id=identifier.generate_uuid(),
                         eventTime=timestamp.get_utc_now(),
                         initiatorId=identifier.generate_uuid(),
                         action='read',
                         targetId=identifier.generate_uuid(),
                         observerId=identifier.generate_uuid(),
                         outcome='success')
        self.assertEqual(ev.is_valid(), True)

        ev = event.Event(eventType='activity',
                         id=identifier.generate_uuid(),
                         eventTime=timestamp.get_utc_now(),
                         initiator=resource.Resource(typeURI='storage'),
                         action='read',
                         targetId=identifier.generate_uuid(),
                         observer=resource.Resource(id='target'),
                         outcome='success')
        self.assertEqual(ev.is_valid(), True)

    def test_event_unique(self):
        ev = event.Event(eventType='activity',
                         initiator=resource.Resource(typeURI='storage'),
                         action='read',
                         target=resource.Resource(typeURI='storage'),
                         observer=resource.Resource(id='target'),
                         outcome='success')
        time.sleep(1)
        ev2 = event.Event(eventType='activity',
                          initiator=resource.Resource(typeURI='storage'),
                          action='read',
                          target=resource.Resource(typeURI='storage'),
                          observer=resource.Resource(id='target'),
                          outcome='success')
        self.assertNotEqual(ev.id, ev2.id)
        self.assertNotEqual(ev.eventTime, ev2.eventTime)

    def test_event_resource_shortform_not_self(self):
        self.assertRaises(ValueError,
                          lambda: event.Event(
                              eventType='activity',
                              initiator=resource.Resource(typeURI='storage'),
                              action='read',
                              target=resource.Resource(id='target'),
                              observer=resource.Resource(id='target'),
                              outcome='success'))
        self.assertRaises(ValueError,
                          lambda: event.Event(
                              eventType='activity',
                              initiator=resource.Resource(id='initiator'),
                              action='read',
                              target=resource.Resource(typeURI='storage'),
                              observer=resource.Resource(id='target'),
                              outcome='success'))

    def _create_none_validator_descriptor(self):
        class Owner(object):
            x = cadftype.ValidatorDescriptor(uuid.uuid4().hex)

        owner = Owner()
        owner.x = None

    def test_invalid_value_descriptor(self):
        """Test setting a ValidatorDescriptor to None results in ValueError"""

        self.assertRaises(ValueError, self._create_none_validator_descriptor)

    def test_cadfabstracttype_attribute_error(self):
        """Test an invalid CADFAbstractType attribute is set returns False"""

        h = host.Host(id=identifier.generate_uuid(),
                      address='192.168.0.1',
                      agent='client',
                      platform='AIX')
        self.assertEqual(h._isset(uuid.uuid4().hex), False)



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.

import six

from pycadf import cadftaxonomy


def convert_req_action(method, details=None):
    """Maps standard HTTP methods to equivalent CADF action

    :param method: HTTP request method
    :param details: Extra details to append to action.
    """

    mapping = {'get': cadftaxonomy.ACTION_READ,
               'head': cadftaxonomy.ACTION_READ,
               'post': cadftaxonomy.ACTION_CREATE,
               'put': cadftaxonomy.ACTION_UPDATE,
               'delete': cadftaxonomy.ACTION_DELETE,
               'patch': cadftaxonomy.ACTION_UPDATE,
               'options': cadftaxonomy.ACTION_READ,
               'trace': 'capture'}

    action = None
    if isinstance(method, six.string_types):
        action = mapping.get(method.lower())
        if action and isinstance(details, six.string_types):
            action += '/%s' % details
    return action or cadftaxonomy.UNKNOWN



# Copyright 2013 IBM Corp.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.

import six


def mask_value(value, s_percent=0.125):
    """Obfuscate a given string to show only a percentage of leading
    and trailing characters.

    :param s_percent: The percentage (in decimal) of characters to replace
    """
    if isinstance(value, six.string_types):
        visible = (32 if int(round(len(value) * s_percent)) > 32
                   else int(round(len(value) * s_percent)))
        return value[:visible] + " xxxxxxxx " + value[-visible:]
    return value



# Copyright (c) 2013 IBM Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.

import six

from pycadf import cadftype

TYPE_URI_ENDPOINT = cadftype.CADF_VERSION_1_0_0 + 'endpoint'

ENDPOINT_KEYNAME_URL = "url"
ENDPOINT_KEYNAME_NAME = "name"
ENDPOINT_KEYNAME_PORT = "port"

ENDPOINT_KEYNAMES = [ENDPOINT_KEYNAME_URL,
                     ENDPOINT_KEYNAME_NAME,
                     ENDPOINT_KEYNAME_PORT]


class Endpoint(cadftype.CADFAbstractType):

    url = cadftype.ValidatorDescriptor(
        ENDPOINT_KEYNAME_URL, lambda x: isinstance(x, six.string_types))
    name = cadftype.ValidatorDescriptor(
        ENDPOINT_KEYNAME_NAME, lambda x: isinstance(x, six.string_types))
    port = cadftype.ValidatorDescriptor(
        ENDPOINT_KEYNAME_PORT, lambda x: isinstance(x, six.string_types))

    def __init__(self, url, name=None, port=None):
        """Create Endpoint data type

        :param url: address of endpoint
        :param name: name of endpoint
        :param port: port of endpoint
        """

        # ENDPOINT.url
        setattr(self, ENDPOINT_KEYNAME_URL, url)
        # ENDPOINT.name
        if name is not None:
            setattr(self, ENDPOINT_KEYNAME_NAME, name)
        # ENDPOINT.port
        if port is not None:
            setattr(self, ENDPOINT_KEYNAME_PORT, port)

    # TODO(mrutkows): validate this cadf:ENDPOINT type against schema
    def is_valid(self):
        """Validation to ensure Endpoint required attributes are set.
        """
        return self._isset(ENDPOINT_KEYNAME_URL)



# Copyright 2013 IBM Corp.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.

"""Test base classes."""

import os.path

import fixtures
from oslo_config import cfg
from oslotest import moxstubout
import testtools


class TestCase(testtools.TestCase):

    def setUp(self):
        super(TestCase, self).setUp()
        self.tempdir = self.useFixture(fixtures.TempDir())
        moxfixture = self.useFixture(moxstubout.MoxStubout())
        self.mox = moxfixture.mox
        self.stubs = moxfixture.stubs
        cfg.CONF([], project='pycadf')

    def path_get(self, project_file=None):
        root = os.path.abspath(os.path.join(os.path.dirname(__file__),
                                            '..',
                                            '..',
                                            )
                               )
        if project_file:
            return os.path.join(root, project_file)
        else:
            return root

    def temp_config_file_path(self, name='api_audit_map.conf'):
        return os.path.join(self.tempdir.path, name)

    def tearDown(self):
        cfg.CONF.reset()
        super(TestCase, self).tearDown()



# Copyright 2013 IBM Corp.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.

from pycadf import cadftype
from pycadf import event

ERROR_UNKNOWN_EVENTTYPE = 'Unknown CADF EventType requested on factory method'


class EventFactory(object):
    """Factory class to create different required attributes for
       the following CADF event types:
       'activity': for tracking any interesting system activities for audit
       'monitor': Events that carry Metrics and Measurements and support
       standards such as NIST
       'control': For audit events that are based upon (security) policies
       and reflect some policy decision.
    """
    def new_event(self, eventType=cadftype.EVENTTYPE_ACTIVITY, **kwargs):
        """Create new event

        :param eventType: eventType of event. Defaults to 'activity'
        """

        # for now, construct a base ('activity') event as the default
        event_val = event.Event(**kwargs)

        if not cadftype.is_valid_eventType(eventType):
            raise ValueError(ERROR_UNKNOWN_EVENTTYPE)

        event_val.eventType = eventType

        # TODO(mrutkows): CADF is only being used for basic
        # 'activity' auditing (on APIs), An IF-ELIF will
        # become more meaningful as we add support for other
        # event types.
        # elif eventType == cadftype.EVENTTYPE_MONITOR:
        #    # TODO(mrutkows): If we add support for standard (NIST)
        #    # monitoring messages, we will would have a "monitor"
        #    # subclass of the CADF Event type and create it here
        #    event_val.set_eventType(cadftype.EVENTTYPE_MONITOR)
        # elif eventType == cadftype.EVENTTYPE_CONTROL:
        #    # TODO(mrutkows): If we add support for standard (NIST)
        #    # monitoring messages, we will would have a "control"
        #    # subclass of the CADF Event type and create it here
        #    event_val.set_eventType(cadftype.EVENTTYPE_CONTROL)
        return event_val



# Copyright 2013 IBM Corp.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.

import six

from pycadf import cadftype
from pycadf import identifier

# Geolocation types can appear outside a cadf:Event record context, in these
# cases a typeURI may be used to identify the cadf:Geolocation data type.
TYPE_URI_GEOLOCATION = cadftype.CADF_VERSION_1_0_0 + 'geolocation'

GEO_KEYNAME_ID = "id"
GEO_KEYNAME_LATITUDE = "latitude"
GEO_KEYNAME_LONGITUDE = "longitude"
GEO_KEYNAME_ELEVATION = "elevation"
GEO_KEYNAME_ACCURACY = "accuracy"
GEO_KEYNAME_CITY = "city"
GEO_KEYNAME_STATE = "state"
GEO_KEYNAME_REGIONICANN = "regionICANN"
# GEO_KEYNAME_ANNOTATIONS = "annotations"

GEO_KEYNAMES = [GEO_KEYNAME_ID,
                GEO_KEYNAME_LATITUDE,
                GEO_KEYNAME_LONGITUDE,
                GEO_KEYNAME_ELEVATION,
                GEO_KEYNAME_ACCURACY,
                GEO_KEYNAME_CITY,
                GEO_KEYNAME_STATE,
                GEO_KEYNAME_REGIONICANN
                # GEO_KEYNAME_ANNOTATIONS
                ]


class Geolocation(cadftype.CADFAbstractType):

    id = cadftype.ValidatorDescriptor(GEO_KEYNAME_ID,
                                      lambda x: identifier.is_valid(x))
    # TODO(mrutkows): we may want to do more validation to make
    # sure numeric range represented by string is valid
    latitude = cadftype.ValidatorDescriptor(GEO_KEYNAME_LATITUDE,
                                            lambda x: isinstance(
                                                x, six.string_types))
    longitude = cadftype.ValidatorDescriptor(GEO_KEYNAME_LONGITUDE,
                                             lambda x: isinstance(
                                                 x, six.string_types))
    elevation = cadftype.ValidatorDescriptor(GEO_KEYNAME_ELEVATION,
                                             lambda x: isinstance(
                                                 x, six.string_types))
    accuracy = cadftype.ValidatorDescriptor(GEO_KEYNAME_ACCURACY,
                                            lambda x: isinstance(
                                                x, six.string_types))
    city = cadftype.ValidatorDescriptor(GEO_KEYNAME_CITY,
                                        lambda x: isinstance(
                                            x, six.string_types))
    state = cadftype.ValidatorDescriptor(GEO_KEYNAME_STATE,
                                         lambda x: isinstance(
                                             x, six.string_types))
    regionICANN = cadftype.ValidatorDescriptor(
        GEO_KEYNAME_REGIONICANN,
        lambda x: isinstance(x, six.string_types))

    def __init__(self, id=None, latitude=None, longitude=None,
                 elevation=None, accuracy=None, city=None, state=None,
                 regionICANN=None):
        """Create Geolocation data type

        :param id: id of geolocation
        :param latitude: latitude of geolocation
        :param longitude: longitude of geolocation
        :param elevation: elevation of geolocation in meters
        :param accuracy: accuracy of geolocation in meters
        :param city: city of geolocation
        :param state: state/province of geolocation
        :param regionICANN: region of geolocation (ie. country)
        """

        # Geolocation.id
        if id is not None:
            setattr(self, GEO_KEYNAME_ID, id)

        # Geolocation.latitude
        if latitude is not None:
            setattr(self, GEO_KEYNAME_LATITUDE, latitude)

        # Geolocation.longitude
        if longitude is not None:
            setattr(self, GEO_KEYNAME_LONGITUDE, longitude)

        # Geolocation.elevation
        if elevation is not None:
            setattr(self, GEO_KEYNAME_ELEVATION, elevation)

        # Geolocation.accuracy
        if accuracy is not None:
            setattr(self, GEO_KEYNAME_ACCURACY, accuracy)

        # Geolocation.city
        if city is not None:
            setattr(self, GEO_KEYNAME_CITY, city)

        # Geolocation.state
        if state is not None:
            setattr(self, GEO_KEYNAME_STATE, state)

        # Geolocation.regionICANN
        if regionICANN is not None:
            setattr(self, GEO_KEYNAME_REGIONICANN, regionICANN)

    # TODO(mrutkows): add mechanism for annotations, OpenStack may choose
    # not to support this "extension mechanism" and is not required (and not
    # critical in many audit contexts)
    def set_annotations(self, value):
        raise NotImplementedError()
        # setattr(self, GEO_KEYNAME_ANNOTATIONS, value)

    # self validate cadf:Geolocation type
    def is_valid(self):
        return True



# Copyright 2013 IBM Corp.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.

import six

from pycadf import cadftype
from pycadf import identifier

# Metric types can appear outside a cadf:Event record context, in these cases
# a typeURI may be used to identify the cadf:Metric data type.
TYPE_URI_METRIC = cadftype.CADF_VERSION_1_0_0 + 'metric'

METRIC_KEYNAME_METRICID = "metricId"
METRIC_KEYNAME_UNIT = "unit"
METRIC_KEYNAME_NAME = "name"
# METRIC_KEYNAME_ANNOTATIONS = "annotations"

METRIC_KEYNAMES = [METRIC_KEYNAME_METRICID,
                   METRIC_KEYNAME_UNIT,
                   METRIC_KEYNAME_NAME
                   # METRIC_KEYNAME_ANNOTATIONS
                   ]


class Metric(cadftype.CADFAbstractType):

    metricId = cadftype.ValidatorDescriptor(METRIC_KEYNAME_METRICID,
                                            lambda x: identifier.is_valid(x))
    unit = cadftype.ValidatorDescriptor(METRIC_KEYNAME_UNIT,
                                        lambda x: isinstance(x,
                                                             six.string_types))
    name = cadftype.ValidatorDescriptor(METRIC_KEYNAME_NAME,
                                        lambda x: isinstance(x,
                                                             six.string_types))

    def __init__(self, metricId=None, unit=None, name=None):
        """Create metric data type

        :param metricId: id of metric. uuid generated if not provided
        :param unit: unit of metric
        :param name: name of metric
        """
        # Metric.id
        setattr(self, METRIC_KEYNAME_METRICID,
                metricId or identifier.generate_uuid())

        # Metric.unit
        if unit is not None:
            setattr(self, METRIC_KEYNAME_UNIT, unit)

        # Metric.name
        if name is not None:
            setattr(self, METRIC_KEYNAME_NAME, name)

    # TODO(mrutkows): add mechanism for annotations, OpenStack may choose
    # not to support this "extension mechanism" and is not required (and not
    # critical in many audit contexts)
    def set_annotations(self, value):
        raise NotImplementedError()
        # setattr(self, METRIC_KEYNAME_ANNOTATIONS, value)

    # self validate cadf:Metric type against schema
    def is_valid(self):
        """Validation to ensure Metric required attributes are set.
        """
        # Existence test, id, and unit attributes must both exist
        return (
            self._isset(METRIC_KEYNAME_METRICID) and
            self._isset(METRIC_KEYNAME_UNIT)
        )



# Copyright 2013 OpenStack LLC
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.

import uuid

from pycadf.tests import base
from pycadf import utils


class TestUtils(base.TestCase):
    def test_mask_value(self):
        value = str(uuid.uuid4())
        m_percent = 0.125
        obfuscate = utils.mask_value(value, m_percent)
        visible = int(round(len(value) * m_percent))
        self.assertEqual(value[:visible], obfuscate[:visible])
        self.assertNotEqual(value[:visible + 1], obfuscate[:visible + 1])
        self.assertEqual(value[-visible:], obfuscate[-visible:])
        self.assertNotEqual(value[-visible - 1:], obfuscate[-visible - 1:])

    def test_mask_value_nonstring(self):
        value = 12

        # If a non-string parameter is given to mask_value(), the non-string
        # parameter is returned unmodified.
        obfuscate = utils.mask_value(value)
        self.assertEqual(value, obfuscate)



# Copyright (c) 2013 IBM Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.

import six

from pycadf import cadftype
from pycadf import utils

TYPE_URI_CRED = cadftype.CADF_VERSION_1_0_0 + 'credential'

CRED_KEYNAME_TYPE = "type"
CRED_KEYNAME_TOKEN = "token"

CRED_KEYNAMES = [CRED_KEYNAME_TYPE,
                 CRED_KEYNAME_TOKEN]


FED_CRED_KEYNAME_IDENTITY_PROVIDER = "identity_provider"
FED_CRED_KEYNAME_USER = "user"
FED_CRED_KEYNAME_GROUPS = "groups"

FED_CRED_KEYNAMES = CRED_KEYNAMES + [FED_CRED_KEYNAME_IDENTITY_PROVIDER,
                                     FED_CRED_KEYNAME_USER,
                                     FED_CRED_KEYNAME_GROUPS]


class Credential(cadftype.CADFAbstractType):
    type = cadftype.ValidatorDescriptor(
        CRED_KEYNAME_TYPE,
        lambda x: isinstance(x, six.string_types))
    token = cadftype.ValidatorDescriptor(
        CRED_KEYNAME_TOKEN,
        lambda x: isinstance(x, six.string_types))

    def __init__(self, token, type=None):
        """Create Credential data type

        :param token: identity or security token
        :param type: type of credential (ie. identity token)
        """

        # Credential.token
        setattr(self, CRED_KEYNAME_TOKEN, utils.mask_value(token))

        # Credential.type
        if type is not None:
            setattr(self, CRED_KEYNAME_TYPE, type)

    # TODO(mrutkows): validate this cadf:Credential type against schema
    def is_valid(self):
        """Validation to ensure Credential required attributes are set."""
        # TODO(mrutkows): validate specific attribute type/format
        return self._isset(CRED_KEYNAME_TOKEN)


class FederatedCredential(Credential):
    identity_provider = cadftype.ValidatorDescriptor(
        FED_CRED_KEYNAME_IDENTITY_PROVIDER,
        lambda x: isinstance(x, six.string_types))
    user = cadftype.ValidatorDescriptor(
        FED_CRED_KEYNAME_USER,
        lambda x: isinstance(x, six.string_types))
    groups = cadftype.ValidatorDescriptor(
        FED_CRED_KEYNAME_GROUPS,
        lambda x: isinstance(x, list))

    def __init__(self, token, type, identity_provider, user, groups):
        super(FederatedCredential, self).__init__(
            token=token,
            type=type)

        # FederatedCredential.identity_provider
        setattr(self, FED_CRED_KEYNAME_IDENTITY_PROVIDER, identity_provider)

        # FederatedCredential.user
        setattr(self, FED_CRED_KEYNAME_USER, user)

        # FederatedCredential.groups
        setattr(self, FED_CRED_KEYNAME_GROUPS, groups)

    def is_valid(self):
        """Validation to ensure Credential required attributes are set."""
        return (
            super(FederatedCredential, self).is_valid()
            and self._isset(CRED_KEYNAME_TYPE)
            and self._isset(FED_CRED_KEYNAME_IDENTITY_PROVIDER)
            and self._isset(FED_CRED_KEYNAME_USER)
            and self._isset(FED_CRED_KEYNAME_GROUPS))



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.

from pycadf import cadftaxonomy
from pycadf.helper import api
from pycadf.tests import base


class TestApiHelper(base.TestCase):
    def test_convert_req_action(self):
        self.assertEqual(cadftaxonomy.ACTION_READ,
                         api.convert_req_action('get'))
        self.assertEqual(cadftaxonomy.ACTION_CREATE,
                         api.convert_req_action('POST'))
        self.assertEqual(cadftaxonomy.ACTION_DELETE,
                         api.convert_req_action('deLetE'))

    def test_convert_req_action_invalid(self):
        self.assertEqual(cadftaxonomy.UNKNOWN, api.convert_req_action(124))
        self.assertEqual(cadftaxonomy.UNKNOWN, api.convert_req_action('blah'))

    def test_convert_req_action_with_details(self):
        detail = 'compute/instance'
        self.assertEqual(cadftaxonomy.ACTION_READ + '/%s' % detail,
                         api.convert_req_action('GET', detail))
        self.assertEqual(cadftaxonomy.ACTION_DELETE + '/%s' % detail,
                         api.convert_req_action('DELETE', detail))

    def test_convert_req_action_with_details_invalid(self):
        detail = 123
        self.assertEqual(cadftaxonomy.ACTION_READ,
                         api.convert_req_action('GET', detail))
        self.assertEqual(cadftaxonomy.ACTION_DELETE,
                         api.convert_req_action('DELETE', detail))



# Copyright 2013 IBM Corp.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.

import datetime

import pytz
import six

TIME_FORMAT = "%Y-%m-%dT%H:%M:%S.%f%z"


def get_utc_now(timezone=None):
    """Return the current UTC time.

    :param timezone: an optional timezone param to offset time to.
    """
    utc_datetime = pytz.utc.localize(datetime.datetime.utcnow())
    if timezone is not None:
        try:
            utc_datetime = utc_datetime.astimezone(pytz.timezone(timezone))
        except Exception:
            utc_datetime.strftime(TIME_FORMAT)
    return utc_datetime.strftime(TIME_FORMAT)


# TODO(mrutkows): validate any cadf:Timestamp (type) record against
# CADF schema
def is_valid(value):
    """Validation to ensure timestamp is a string.
    """
    if not isinstance(value, six.string_types):
        raise ValueError('Timestamp should be a String')

    return True



# Copyright 2013 IBM Corp.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.

import six

from pycadf import cadftype


class Path(cadftype.CADFAbstractType):

    def set_path_absolute(self):
        # TODO(mrutkows): validate absolute path format, else Type error
        raise NotImplementedError()

    def set_path_relative(self):
        # TODO(mrutkows); validate relative path format, else Type error
        raise NotImplementedError()

    # TODO(mrutkows): validate any cadf:Path (type) record against CADF schema
    @staticmethod
    def is_valid(value):
        if not isinstance(value, six.string_types):
            raise TypeError

        return True



# Copyright 2013 IBM Corp.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.

from pycadf import cadftype
from pycadf import identifier
from pycadf import metric
from pycadf import resource

MEASUREMENT_KEYNAME_RESULT = "result"
MEASUREMENT_KEYNAME_METRIC = "metric"
MEASUREMENT_KEYNAME_METRICID = "metricId"
MEASUREMENT_KEYNAME_CALCBY = "calculatedBy"

MEASUREMENT_KEYNAMES = [MEASUREMENT_KEYNAME_RESULT,
                        MEASUREMENT_KEYNAME_METRICID,
                        MEASUREMENT_KEYNAME_METRIC,
                        MEASUREMENT_KEYNAME_CALCBY]


class Measurement(cadftype.CADFAbstractType):

    result = cadftype.ValidatorDescriptor(MEASUREMENT_KEYNAME_RESULT)
    metric = cadftype.ValidatorDescriptor(
        MEASUREMENT_KEYNAME_METRIC, lambda x: isinstance(x, metric.Metric))
    metricId = cadftype.ValidatorDescriptor(MEASUREMENT_KEYNAME_METRICID,
                                            lambda x: identifier.is_valid(x))
    calculatedBy = cadftype.ValidatorDescriptor(
        MEASUREMENT_KEYNAME_CALCBY,
        (lambda x: isinstance(x, resource.Resource) and x.is_valid()))

    def __init__(self, result=None, metric=None, metricId=None,
                 calculatedBy=None):
        """Create Measurement data type

        :param result: value of measurement
        :param metric: Metric data type of current measurement
        :param metricId: id of Metric data type of current measurement
        :param calculatedBy: Resource that calculated measurement
        """
        # Measurement.result
        if result is not None:
            setattr(self, MEASUREMENT_KEYNAME_RESULT, result)

        # Measurement.metricId
        if metricId is not None:
            setattr(self, MEASUREMENT_KEYNAME_METRICID, metricId)

        # Measurement.metric
        if metric is not None:
            setattr(self, MEASUREMENT_KEYNAME_METRIC, metric)

        # Measurement.calculaedBy
        if calculatedBy is not None:
            setattr(self, MEASUREMENT_KEYNAME_CALCBY, calculatedBy)

    # self validate this cadf:Measurement type against schema
    def is_valid(self):
        """Validation to ensure Measurement required attributes are set.
        """
        return (self._isset(MEASUREMENT_KEYNAME_RESULT) and
                (self._isset(MEASUREMENT_KEYNAME_METRIC) ^
                 self._isset(MEASUREMENT_KEYNAME_METRICID)))



# Copyright 2013 OpenStack Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import os.path as path

from sphinx import apidoc


# NOTE(gordc): pbr will run Sphinx multiple times when it generates
# documentation. Once for each builder. To run this extension we use the
# 'builder-inited' hook that fires at the beginning of a Sphinx build.
# We use ``run_already`` to make sure apidocs are only generated once
# even if Sphinx is run multiple times.
run_already = False


def run_apidoc(app):
    global run_already
    if run_already:
        return
    run_already = True

    package_dir = path.abspath(path.join(app.srcdir, '..', '..', 'pycadf'))
    source_dir = path.join(app.srcdir, 'api')
    apidoc.main(['apidoc', package_dir, '-f',
                 '-H', 'pyCADF Modules',
                 '-o', source_dir])


def setup(app):
    app.connect('builder-inited', run_apidoc)



# Copyright (c) 2013 IBM Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.

import abc

from oslo_serialization import jsonutils
import six

CADF_SCHEMA_1_0_0 = 'cadf:'
CADF_VERSION_1_0_0 = 'http://schemas.dmtf.org/cloud/audit/1.0/'

# Valid cadf:Event record "types"
EVENTTYPE_ACTIVITY = 'activity'
EVENTTYPE_MONITOR = 'monitor'
EVENTTYPE_CONTROL = 'control'

VALID_EVENTTYPES = frozenset([
    EVENTTYPE_ACTIVITY,
    EVENTTYPE_MONITOR,
    EVENTTYPE_CONTROL
])


def is_valid_eventType(value):
    return value in VALID_EVENTTYPES

# valid cadf:Event record "Reporter" roles
REPORTER_ROLE_OBSERVER = 'observer'
REPORTER_ROLE_MODIFIER = 'modifier'
REPORTER_ROLE_RELAY = 'relay'

VALID_REPORTER_ROLES = frozenset([
    REPORTER_ROLE_OBSERVER,
    REPORTER_ROLE_MODIFIER,
    REPORTER_ROLE_RELAY
])


def is_valid_reporter_role(value):
    return value in VALID_REPORTER_ROLES


class ValidatorDescriptor(object):
    def __init__(self, name, func=None):
        self.name = name
        self.func = func

    def __set__(self, instance, value):
        if value is not None:
            if self.func is not None:
                if self.func(value):
                    instance.__dict__[self.name] = value
                else:
                    raise ValueError('%s failed validation: %s' %
                                     (self.name, self.func))
            else:
                instance.__dict__[self.name] = value
        else:
            raise ValueError('%s must not be None.' % self.name)


@six.add_metaclass(abc.ABCMeta)
class CADFAbstractType(object):
    """The abstract base class for all CADF (complex) data types (classes)."""

    @abc.abstractmethod
    def is_valid(self, value):
        pass

    def as_dict(self):
        """Return dict representation of Event."""
        return jsonutils.to_primitive(self, convert_instances=True)

    def _isset(self, attr):
        """Check to see if attribute is defined."""
        try:
            if isinstance(getattr(self, attr), ValidatorDescriptor):
                return False
            return True
        except AttributeError:
            return False

    # TODO(mrutkows): Eventually, we want to use the OrderedDict (introduced
    # in Python 2.7) type for all CADF classes to store attributes in a
    # canonical form.  Currently, OpenStack/Jenkins requires 2.6 compatibility
    # The reason is that we want to be able to support signing all or parts
    # of the event record and need to guarantee order.
    # def to_ordered_dict(self, value):
    #    pass



# Copyright 2013 IBM Corp.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.

import six


def generate_name_value_tag(name, value):
    """Generate a CADF tag in the format name?value=<value>

    :param name: name of tag
    :param valuue: optional value tag
    """
    if name is None or value is None:
        raise ValueError('Invalid name and/or value. Values cannot be None')

    tag = name + "?value=" + value
    return tag


# TODO(mrutkows): validate any Tag's name?value= format
def is_valid(value):
    """Validation check to ensure proper Tag format
    """
    if not isinstance(value, six.string_types):
        raise TypeError
    return True



# Copyright 2013 IBM Corp.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.

import six

from pycadf import attachment
from pycadf import cadftaxonomy
from pycadf import cadftype
from pycadf import credential
from pycadf import endpoint
from pycadf import geolocation
from pycadf import host
from pycadf import identifier

TYPE_URI_RESOURCE = cadftype.CADF_VERSION_1_0_0 + 'resource'

RESOURCE_KEYNAME_TYPEURI = "typeURI"
RESOURCE_KEYNAME_ID = "id"
RESOURCE_KEYNAME_NAME = "name"
RESOURCE_KEYNAME_DOMAIN = "domain"
RESOURCE_KEYNAME_CRED = "credential"
RESOURCE_KEYNAME_REF = "ref"
RESOURCE_KEYNAME_GEO = "geolocation"
RESOURCE_KEYNAME_GEOID = "geolocationId"
RESOURCE_KEYNAME_HOST = "host"
RESOURCE_KEYNAME_ADDRS = "addresses"
RESOURCE_KEYNAME_ATTACHMENTS = "attachments"

RESOURCE_KEYNAMES = [RESOURCE_KEYNAME_TYPEURI,
                     RESOURCE_KEYNAME_ID,
                     RESOURCE_KEYNAME_NAME,
                     RESOURCE_KEYNAME_DOMAIN,
                     RESOURCE_KEYNAME_CRED,
                     RESOURCE_KEYNAME_REF,
                     RESOURCE_KEYNAME_GEO,
                     RESOURCE_KEYNAME_GEOID,
                     RESOURCE_KEYNAME_HOST,
                     RESOURCE_KEYNAME_ADDRS,
                     RESOURCE_KEYNAME_ATTACHMENTS]


class Resource(cadftype.CADFAbstractType):

    typeURI = cadftype.ValidatorDescriptor(
        RESOURCE_KEYNAME_TYPEURI, lambda x: cadftaxonomy.is_valid_resource(x))
    id = cadftype.ValidatorDescriptor(RESOURCE_KEYNAME_ID,
                                      lambda x: identifier.is_valid(x))
    name = cadftype.ValidatorDescriptor(RESOURCE_KEYNAME_NAME,
                                        lambda x: isinstance(x,
                                                             six.string_types))
    domain = cadftype.ValidatorDescriptor(RESOURCE_KEYNAME_DOMAIN,
                                          lambda x: isinstance(
                                              x, six.string_types))
    credential = cadftype.ValidatorDescriptor(
        RESOURCE_KEYNAME_CRED, (lambda x: isinstance(x, credential.Credential)
                                and x.is_valid()))
    host = cadftype.ValidatorDescriptor(
        RESOURCE_KEYNAME_HOST, lambda x: isinstance(x, host.Host))
    # TODO(mrutkows): validate the "ref" attribute is indeed a URI (format),
    # If it is a URL, we do not need to validate it is accessible/working,
    # for audit purposes this could have been a valid URL at some point
    # in the past or a URL that is only valid within some domain (e.g. a
    # private cloud)
    ref = cadftype.ValidatorDescriptor(RESOURCE_KEYNAME_REF,
                                       lambda x: isinstance(x,
                                                            six.string_types))
    geolocation = cadftype.ValidatorDescriptor(
        RESOURCE_KEYNAME_GEO,
        lambda x: isinstance(x, geolocation.Geolocation))
    geolocationId = cadftype.ValidatorDescriptor(
        RESOURCE_KEYNAME_GEOID, lambda x: identifier.is_valid(x))

    def __init__(self, id=None, typeURI=cadftaxonomy.UNKNOWN, name=None,
                 ref=None, domain=None, credential=None, host=None,
                 geolocation=None, geolocationId=None):
        """Resource data type

        :param id: id of resource
        :param typeURI: typeURI of resource, defaults to 'unknown' if not set
        :param name: name of resource
        :param domain: domain to qualify name of resource
        :param credential: optional security Credential data type
        :param host: optional Host data type information relating to resource
        :param geolocation: optional CADF Geolocation of resource
        :param geolocationId: optional id of CADF Geolocation for resource
        """

        # Resource.id
        setattr(self, RESOURCE_KEYNAME_ID, id or identifier.generate_uuid())

        # Resource.typeURI
        if (getattr(self, RESOURCE_KEYNAME_ID) != "target" and
                getattr(self, RESOURCE_KEYNAME_ID) != "initiator"):
            setattr(self, RESOURCE_KEYNAME_TYPEURI, typeURI)

        # Resource.name
        if name is not None:
            setattr(self, RESOURCE_KEYNAME_NAME, name)

        # Resource.ref
        if ref is not None:
            setattr(self, RESOURCE_KEYNAME_REF, ref)

        # Resource.domain
        if domain is not None:
            setattr(self, RESOURCE_KEYNAME_DOMAIN, domain)

        # Resource.credential
        if credential is not None:
            setattr(self, RESOURCE_KEYNAME_CRED, credential)

        # Resource.host
        if host is not None:
            setattr(self, RESOURCE_KEYNAME_HOST, host)

        # Resource.geolocation
        if geolocation is not None:
            setattr(self, RESOURCE_KEYNAME_GEO, geolocation)

        # Resource.geolocationId
        if geolocationId:
            setattr(self, RESOURCE_KEYNAME_GEOID, geolocationId)

    # Resource.address
    def add_address(self, addr):
        """Add CADF endpoints to Resource

        :param addr: CADF Endpoint to add to Resource
        """
        if (addr is not None and isinstance(addr, endpoint.Endpoint)):
            if addr.is_valid():
                # Create the list of Endpoints if needed
                if not hasattr(self, RESOURCE_KEYNAME_ADDRS):
                    setattr(self, RESOURCE_KEYNAME_ADDRS, list())

                addrs = getattr(self, RESOURCE_KEYNAME_ADDRS)
                addrs.append(addr)
            else:
                raise ValueError('Invalid endpoint')
        else:
            raise ValueError('Invalid endpoint. Value must be an Endpoint')

    # Resource.attachments
    def add_attachment(self, attach_val):
        """Add CADF attachment to Resource

        :param attach_val: CADF Attachment to add to Resource
        """
        if (attach_val is not None
                and isinstance(attach_val, attachment.Attachment)):
            if attach_val.is_valid():
                # Create the list of Attachments if needed
                if not hasattr(self, RESOURCE_KEYNAME_ATTACHMENTS):
                    setattr(self, RESOURCE_KEYNAME_ATTACHMENTS, list())

                attachments = getattr(self, RESOURCE_KEYNAME_ATTACHMENTS)
                attachments.append(attach_val)
            else:
                raise ValueError('Invalid attachment')
        else:
            raise ValueError('Invalid attachment. Value must be an Attachment')

    # self validate this cadf:Resource type against schema
    def is_valid(self):
        """Validation to ensure Resource required attributes are set
        """
        return (self._isset(RESOURCE_KEYNAME_ID) and
                (self._isset(RESOURCE_KEYNAME_TYPEURI) or
                 ((getattr(self, RESOURCE_KEYNAME_ID) == "target" or
                   getattr(self, RESOURCE_KEYNAME_ID) == "initiator") and
                  len(vars(self).keys()) == 1)))
        # TODO(mrutkows): validate the Resource's attribute types



# Copyright 2013 IBM Corp.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.
import hashlib
import uuid

from debtcollector import removals
from oslo_config import cfg

CONF = cfg.CONF
opts = [
    cfg.StrOpt('namespace',
               default='openstack',
               help='namespace prefix for generated id'),
]
CONF.register_opts(opts, group='audit')


AUDIT_NS = None
if CONF.audit.namespace:
    md5_hash = hashlib.md5(CONF.audit.namespace.encode('utf-8'))
    AUDIT_NS = uuid.UUID(md5_hash.hexdigest())


def generate_uuid():
    """Generate a CADF identifier
    """
    if AUDIT_NS:
        return str(uuid.uuid5(AUDIT_NS, str(uuid.uuid4())))
    return str(uuid.uuid4())


@removals.remove
def norm_ns(str_id):
    """Apply a namespace to the identifier """
    prefix = CONF.audit.namespace + ':' if CONF.audit.namespace else ''
    return prefix + str_id


def is_valid(value):
    """Validation to ensure Identifier is correct.
    """
    if value in ['target', 'initiator', 'observer']:
        return True
    try:
        uuid.UUID(value)
    except ValueError:
        return False
    else:
        return True
